# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
import re

from django.core import exceptions
from django.db import models
from django.utils.translation import ugettext_lazy as _


partial_date_re = re.compile(
    r'^(?P<year>\d{4})(?:-(?P<month>\d{1,2}))?(?:-(?P<day>\d{1,2}))?$'
)


class PartialDate(object):
    YEAR = 0
    MONTH = 1
    DAY = 2

    _date = None
    _precision = None

    DATE_FORMATS = {
        YEAR: '%Y',
        MONTH: '%Y-%m',
        DAY: '%Y-%m-%d'
    }

    def __init__(self, date, precision=DAY):
        if isinstance(date, str):
            date, precision =  PartialDate.parseDate(date)

        self.date = date
        self.precision = precision

    def __repr__(self):
        return "" if not self._date else self._date.strftime(self.DATE_FORMATS[self._precision])

    def format(self, precision_year=None, precision_month=None, precision_day=None):
        if self.precisionYear():
            format = precision_year
        elif self.precisionMonth():
            format = precision_month
        else:
            format = precision_day
        return "" if not self._date else self._date.strftime(format)

    @property
    def date(self):
        return self._date

    @date.setter
    def date(self, value):
        if not isinstance(value, datetime.date):
            raise exceptions.ValidationError(
                _("%(value)s is not datetime.date instance"),
                params={'value': value},
            )
        self._date = value

    @property
    def precision(self):
        return self._precision

    @precision.setter
    def precision(self, value):
        self._precision = value if value in (self.YEAR, self.MONTH, self.DAY) else self.DAY
        if self._precision == self.MONTH:
            self._date.replace(day=1)
        if self._precision == self.YEAR:
            self._date.replace(month=1, day=1)

    def precisionYear(self):
        return self.precision == self.YEAR

    def precisionMonth(self):
        return self.precision == self.MONTH

    def precisionDay(self):
        return self.precision == self.DAY

    @staticmethod
    def parseDate(value):
        """
        Returns a tuple (datetime.date, precision) from a string formatted as YYYY, YYYY-MM, YYYY-MM-DD.
        """
        match = partial_date_re.match(value)

        try:
            match_dict = match.groupdict()
            kw = {k: int(v) if v else 1 for k, v in iter(match_dict.items())}

            precision = PartialDate.DAY if match_dict["day"] else \
                        PartialDate.MONTH if match_dict["month"] else \
                        PartialDate.YEAR
            return (datetime.date(**kw), precision)
        except (AttributeError, ValueError):
            raise exceptions.ValidationError(
                _("'%(value)s' is not a valid date string (YYYY, YYYY-MM, YYYY-MM-DD)"),
                params={'value': value}
            )

    def __eq__(self, other):
        if isinstance(other, PartialDate):
            return self.date == other.date and self.precision == other.precision
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, PartialDate):
            return self.__ge__(other) and not self.__eq__(other)
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, PartialDate):
            return self.date >= other.date and self.precision >= other.precision
        else:
            return NotImplemented


class PartialDateField(models.Field):
    """
    A django model field for storing partial dates.
    Accepts None, a partial_date.PartialDate object,
    or a formatted string such as YYYY, YYYY-MM, YYYY-MM-DD.
    In the database it saves the date in a column of type DateTimeField
    and uses the seconds to save the level of precision.
    """

    def get_internal_type(self):
        return "DateTimeField"

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        return PartialDate(value.date(), value.second)

    def to_python(self, value):
        if value is None:
            return value

        if isinstance(value, PartialDate):
            return value

        if isinstance(value, str):
            return PartialDate(value)

        raise exceptions.ValidationError(
            _("'%(name)s' value must be a PartialDate instance, "
                "a valid partial date string (YYYY, YYYY-MM, YYYY-MM-DD) "
                "or None, not '%(value)s'"),
            params={'name': self.name, 'value': value},
        )

    def get_prep_value(self, value):
        if value in (None, ''):
            return None
        partial_date = self.to_python(value)
        date = partial_date.date
        return datetime.datetime(date.year, date.month, date.day, second=partial_date.precision)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import date
from django.test import TestCase
from partial_date import PartialDate, PartialDateField


class PartialDateTestCase(TestCase):
    def test_init_with_string(self):
        self.assertEqual(PartialDate("2000").date, date(year=2000, month=1, day=1))
        self.assertEqual(PartialDate("2000-02").date, date(year=2000, month=2, day=1))
        self.assertEqual(PartialDate("2000-03-04").date, date(year=2000, month=3, day=4))

        self.assertEqual(PartialDate("2000").precision, PartialDate.YEAR)
        self.assertEqual(PartialDate("2000-02").precision, PartialDate.MONTH)
        self.assertEqual(PartialDate("2000-03-04").precision, PartialDate.DAY)

    def test_init_with_date(self):
        self.assertEqual(PartialDate(date(year=2000, month=3, day=4), precision=PartialDate.DAY).__repr__(), "2000-03-04")
        self.assertEqual(PartialDate(date(year=2000, month=3, day=4), precision=PartialDate.MONTH).__repr__(), "2000-03")
        self.assertEqual(PartialDate(date(year=2000, month=3, day=4), precision=PartialDate.YEAR).__repr__(), "2000")

    def test_eq(self):
        self.assertTrue(PartialDate("2000") == PartialDate("2000"))
        self.assertFalse(PartialDate("2001") == PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") == PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") == PartialDate("2000-01-01"))

    def test_ne(self):
        self.assertFalse(PartialDate("2000") != PartialDate("2000"))
        self.assertTrue(PartialDate("2001") != PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") != PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") != PartialDate("2000-01-01"))

    def test_gt(self):
        self.assertFalse(PartialDate("2000") > PartialDate("2000"))
        self.assertTrue(PartialDate("2001") > PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") > PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") > PartialDate("2000-01-01"))

    def test_lt(self):
        self.assertFalse(PartialDate("2000") < PartialDate("2000"))
        self.assertFalse(PartialDate("2001") < PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") < PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") < PartialDate("2000-01-01"))

    def test_ge(self):
        self.assertTrue(PartialDate("2000") >= PartialDate("2000"))
        self.assertTrue(PartialDate("2001") >= PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") >= PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") >= PartialDate("2000-01-01"))

    def test_le(self):
        self.assertTrue(PartialDate("2000") <= PartialDate("2000"))
        self.assertFalse(PartialDate("2001") <= PartialDate("2000"))
        self.assertFalse(PartialDate("2000-01") <= PartialDate("2000"))
        self.assertTrue(PartialDate("2000-01") <= PartialDate("2000-01-01"))

    def test_format(self):
        format = ('%Y', '%m/%Y', '%m/%d/%Y')
        self.assertEqual(PartialDate("2000-03-04").format(*format), "03/04/2000")
        self.assertEqual(PartialDate("2000-03").format(*format), "03/2000")
        self.assertEqual(PartialDate("2000").format(*format), "2000")



import os
import sys

from setuptools import setup

from codecs import open

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)

# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of r-django-partial-date requires Python {}.{}, but you're trying to
install it on Python {}.{}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have pip >= 9.0 and setuptools >= 24.2, then try again:
    $ python -m pip install --upgrade pip setuptools
    $ python -m pip install django
This will install the latest version of r-django-partial-date which works on your
version of Python.
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)

here = os.path.abspath(os.path.dirname(__file__))


def read(fname):
    path = os.path.join(os.path.dirname(__file__), fname)
    try:
        file = open(path, encoding='utf-8')
    except TypeError:
        file = open(path)
    return file.read()


with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='r-django-partial-date',
    version='1.3.1',
    description='Django custom model field for partial dates with the form YYYY, YYYY-MM, YYYY-MM-DD',
    long_description=read('README.rst'),
    author_email='ramses.mtz96@gmail.com',
    author='Ramses Martinez',
    url='https://github.com/RamsesMartinez/r-django-partial-date',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords=['fields', 'django', 'dates', 'partial'],

    packages=['partial_date'],

    install_requires=[
        'django',
    ],
)
